#include <QApplication>
#include <QStringList>
#include <QTranslator>
#include <QLocale>
#include <QStandardPaths>
#include <QLibraryInfo>
#include <QDir>
#include <fcntl.h>
#include <syslog.h>
#include <QTranslator>
#include <QLocale>

#include <QDebug>
#include <QFile>
#include <QMutex>
#include <QDateTime>
#include <sys/inotify.h>


#include "mainwindow.h"
#include "xatom-helper.h"
#include "common/dbusadaptor.h"
#include "data_warehouse.h"
#include "logic_center.h"

#if 0
void myMessageOutput(QtMsgType type, const QMessageLogContext &context, const QString &msg)
{
    /* 加锁 */
    static QMutex mutex;

    mutex.lock();

    QByteArray localMsg = msg.toLocal8Bit();

    QString strMsg("");
    switch (type)
    {
    case QtDebugMsg:
    default:
        strMsg = QString("Debug:");
        break;
    case QtWarningMsg:
        strMsg = QString("Warning:");
        break;
    case QtCriticalMsg:
        strMsg = QString("Critical:");
        break;
    case QtFatalMsg:
        strMsg = QString("Fatal:");
        break;
    }

    /* 设置输出信息格式 */
    QString strDateTime = QDateTime::currentDateTime().toString("yyyy-MM-dd");
    QString strMessage = QString(QString() + "Message:%1").arg(localMsg.constData());

    /* 输出信息至文件中（读写、追加形式）*/
    QString dirPath = "/tmp/kylin-calculator/log";
    QDir dir;
    QFile file;
    if (dir.mkpath(dirPath)) {
        dirPath = dirPath + "/" + strDateTime + ".txt" ;
        file.setFileName(dirPath);
    }

    file.open(QIODevice::ReadWrite | QIODevice::Append);
    QTextStream stream(&file);
    stream << strMessage << "\r\n";
    file.flush();
    file.close();

    // 解锁
    mutex.unlock();
}
#endif

int main(int argc, char *argv[])
{
    //initUkuiLog4qt("kylin-calculator");
    printf("Program start ...\n");

    /* 适配4K屏 */
    #if (QT_VERSION >= QT_VERSION_CHECK(5 , 12 , 0))
        QApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
        QApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
    #endif

    #if (QT_VERSION >= QT_VERSION_CHECK(5 , 14 , 0))
        QApplication::setHighDpiScaleFactorRoundingPolicy(Qt::HighDpiScaleFactorRoundingPolicy::PassThrough);
    #endif

    QApplication a(argc, argv);
    a.setApplicationVersion("1.1.0");
    a.setWindowIcon(QIcon::fromTheme("kylin-calculator"));

    /* 连接DBus服务 */
    QDBusInterface interface(KYLIN_CALCULATOR_SERVICE , KYLIN_CALCULATOR_PATH , KYLIN_CALCULATOR_INTERFACE , QDBusConnection::sessionBus());
    if (interface.isValid()) {
        qDebug() << "showMainWindow";
        interface.call("showMainWindow");
        return 0;
    }

    /* 实现VNC单例 */
    QStringList homePath = QStandardPaths::standardLocations(QStandardPaths::HomeLocation);

    /* 需要给文件锁加一个DISPLAY标识  QtSingleApplication-Name改为kylin-calculator */
    int fd = open(QString(homePath.at(0) + "/.config/kylin-calculator%1.lock").arg(getenv("DISPLAY")).toUtf8().data() , O_WRONLY | O_CREAT | O_TRUNC , S_IRUSR | S_IWUSR);
    if (fd < 0) {
        exit(1);
    }
    if (lockf(fd , F_TLOCK , 0)) {
        syslog(LOG_ERR, "Can't lock single file, kylin-calculator is already running!");
        exit(0);
    }

    /* 加载翻译文件 */
    QString tranPath("/usr/share/kylin-calculator/translations/");
    QString tranQtPath = QLibraryInfo::location(QLibraryInfo::TranslationsPath);
    QTranslator *tran = new QTranslator;
    QTranslator *tranQt = new QTranslator;
    if (tran->load(QLocale() , QString("kylin-calculator") , QString("_") , tranPath)) {
        QApplication::installTranslator(tran);
    } else {
        qDebug() << "Waring : load translation file fail";
    }

    if (tranQt->load(QLocale() , QString("qt") , QString("_") , tranQtPath)) {
        QApplication::installTranslator(tranQt);
    } else {
        qDebug() << "Waring : load qt translation file fail";
    }

    /* 实例 */
    LogicCenter::getInstance();

    /* 添加窗管协议 */
    MotifWmHints hints;
    hints.flags = MWM_HINTS_FUNCTIONS | MWM_HINTS_DECORATIONS;
    hints.functions = MWM_FUNC_ALL;
    hints.decorations = MWM_DECOR_BORDER;
    XAtomHelper::getInstance()->setWindowMotifHint(MainWindow::getInstance()->winId(), hints);

    /* 移动到窗口中间 */
    QRect availableGeometry = qApp->primaryScreen()->availableGeometry();
    MainWindow::getInstance()->move(availableGeometry.width() / 2 - MainWindow::getInstance()->width() / 2 , availableGeometry.height() / 2 - MainWindow::getInstance()->height() / 2);
    MainWindow::getInstance()->show();

    /* 创建DBus服务 */
    DbusAdaptor adaptor;
    Q_UNUSED(adaptor);

    return a.exec();
}
