#include <QDebug>
#include <QTimer>

#include "toolmodel.h"
#include "cal.h"

using namespace std;

ToolModelOutput::ToolModelOutput(QWidget *parent) : QWidget(parent)
{
    /* 初始化组件 */
    this->setWidgetUi();

    /* 设置组件样式 */
    this->setWidgetStyle();
}


// 初始化组件
void ToolModelOutput::setWidgetUi()
{
    /* 汇率刷新 */
    toolLabUpdate  = new QLabel(this);
    toolIconUpdate = new QToolButton(this);
    toolLabTime    = new QLabel(this);
    toolLabRate    = new QLabel(this);

    toolUpdateLayout = new QVBoxLayout(this);
    timeRateLayout   = new QVBoxLayout(this);

    timeRateLayout->addWidget(toolLabTime, 0, Qt::AlignCenter);
    timeRateLayout->addWidget(toolLabRate, 0, Qt::AlignCenter);
    timeRateLayout->setSpacing(0);
    timeRateLayout->setMargin(0);

    toolUpdateLayout->addSpacing(15);
    toolUpdateLayout->addWidget(toolLabUpdate,  0, Qt::AlignCenter);
    toolUpdateLayout->addWidget(toolIconUpdate, 0, Qt::AlignCenter);
    toolUpdateLayout->addLayout(timeRateLayout, 0);
    toolUpdateLayout->addSpacing(15);
    toolUpdateLayout->setMargin(0);

    toolUpdateWid = new QWidget(this);
    toolUpdateWid->setLayout(toolUpdateLayout);
    toolUpdateWid->setSizePolicy(QSizePolicy::Minimum,QSizePolicy::Expanding);

    /* 换算前单位标识 */
    toolLabUnitBef = new QLabel(this);
    toolLabIconBef = new QLabel(this);

    toolLabUnitBef->setFixedWidth(64);

    toolUnitBefLayout = new QHBoxLayout(this);
    toolUnitBefLayout->addWidget(toolLabUnitBef);
    toolUnitBefLayout->addWidget(toolLabIconBef);
    toolUnitBefLayout->addSpacing(16);
    toolUnitBefLayout->setMargin(0);

    toolUnitBefWid = new QPushButton(this);
    toolUnitBefWid->setLayout(toolUnitBefLayout);
    toolUnitBefWid->setSizePolicy(QSizePolicy::Minimum,QSizePolicy::Expanding);

    /* 换算后单位标识 */
    toolLabUnitAft = new QLabel(this);
    toolLabIconAft = new QLabel(this);

    toolLabUnitAft->setFixedWidth(64);

    toolUnitAftLayout = new QHBoxLayout(this);
    toolUnitAftLayout->addWidget(toolLabUnitAft);
    toolUnitAftLayout->addWidget(toolLabIconAft);
    toolUnitAftLayout->addSpacing(16);
    toolUnitAftLayout->setMargin(0);

    toolUnitAftWid = new QPushButton(this);
    toolUnitAftWid->setLayout(toolUnitAftLayout);
    toolUnitAftWid->setSizePolicy(QSizePolicy::Minimum,QSizePolicy::Expanding);

    /* 汇率刷新 */
    toolLabUpdate->setText(tr("Rate update"));
    toolIconUpdate->setIcon(QIcon::fromTheme("transform-rotate", QIcon(":/image/update.png")));
    toolIconUpdate->setIconSize(QSize(16, 16));
    toolIconUpdate->setProperty("isWindowButton", 0x1);    
    toolIconUpdate->setProperty("useIconHighlightEffect", 0x2);     
    toolIconUpdate->setAutoRaise(true); 

    /* 默认汇率 */
    toolDouRate = 0.15;
    toolLabTime->setText("2020.09.03 09:30");
    toolLabRate->setText("1 CNY = 0.15 USD");

    connect(toolIconUpdate , &QToolButton::clicked , this , &ToolModelOutput::updateRate);

    /* 换算前单位标识 */
    toolRateNameBef = tr("Chinese Yuan");
    toolRateSymbBef = "CNY";
    toolLabUnitBef->setText(toolRateNameBef + "\n" + toolRateSymbBef);
    toolLabUnitBef->setAlignment(Qt::AlignCenter);
    toolLabIconBef->setPixmap(QIcon::fromTheme("ukui-down-symbolic").pixmap(16, 16));
    toolLabIconBef->setFixedSize(16, 16);

    /* 换算后单位标识 */
    toolRateNameAft = tr("US Dollar");
    toolRateSymbAft = "USD";
    toolLabUnitAft->setText(toolRateNameAft + "\n" + toolRateSymbAft);
    toolLabUnitAft->setAlignment(Qt::AlignCenter);
    toolLabIconAft->setPixmap(QIcon::fromTheme("ukui-down-symbolic").pixmap(16, 16));
    toolLabIconAft->setFixedSize(16, 16);

    connect(toolUnitBefWid, SIGNAL(clicked()), this, SLOT(unitListBefShow()));
    connect(toolUnitAftWid, SIGNAL(clicked()), this, SLOT(unitListAftShow()));

    // 数据输出界面
    this->toolLabHis = new QLabel(this);
    this->toolLabBef = new QLabel(this);
    this->toolLabAft = new QLabel(this);

    this->toolLabAft->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
    this->toolLabBef->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
    this->toolLabAft->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);

    QFont toolLabHisFont("SourceHanSansCN-Light", 40, 15);
    this->toolLabHis->setAlignment(Qt::AlignRight | Qt::AlignTop);
    this->toolLabHis->setFont(toolLabHisFont);
    this->toolLabHis->setText("");
    this->toolLabHis->setContentsMargins(0, 0, 20, 0);
    this->toolLabHis->show();

    QFont toolLabBefFont("SourceHanSansCN-Normal;", 48, 15);
    this->toolLabBef->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
    this->toolLabBef->setFont(toolLabBefFont);
    this->toolLabBef->setText("0");
    this->toolLabBef->setContentsMargins(0, 0, 20, 0);
    this->toolLabBef->show();

    QFont toolLabAftFont("SourceHanSansCN-Normal", 48, 15);
    this->toolLabAft->setAlignment(Qt::AlignRight | Qt::AlignBottom);
    this->toolLabAft->setFont(toolLabAftFont);
    this->toolLabAft->setText("0");
    this->toolLabAft->setContentsMargins(0, 0, 20, 0);
    this->toolLabAft->show();

    // 将数据输出组件和数据单位列表加入布局
    QGridLayout *toolOutputLayout = new QGridLayout(this);

    toolOutputLayout->addWidget(toolUpdateWid, 0, 0, 1, 1);
    toolOutputLayout->addWidget(toolUnitBefWid, 1, 0, 1, 1);
    toolOutputLayout->addWidget(toolUnitAftWid, 2, 0, 1, 1);
    toolOutputLayout->addWidget(toolLabHis, 0, 1, 1, 3);
    toolOutputLayout->addWidget(toolLabBef, 1, 1, 1, 3);
    toolOutputLayout->addWidget(toolLabAft, 2, 1, 1, 3);


    toolOutputLayout->setSpacing(2);
    toolOutputLayout->setContentsMargins(1, 1, 1, 0);

    this->setLayout(toolOutputLayout);
    //this->setFixedHeight(270);

    // 数据单位列表
    this->unitListBef  = new UnitListWidget(this->parentWidget(), this->toolUnitBefWid);
    this->unitListAft  = new UnitListWidget(this->parentWidget(), this->toolUnitAftWid);

    // QString strUnitList = "人民币,美元,欧元,英镑,日元,韩元,澳大利亚元";
    // initUnitList(strUnitList);

    connect(unitListBef->unitList, SIGNAL(itemClicked(QListWidgetItem*)), this, SLOT(unitListSelect(QListWidgetItem*)));
    connect(unitListAft->unitList, SIGNAL(itemClicked(QListWidgetItem*)), this, SLOT(unitListSelect(QListWidgetItem*)));

    connect(unitListBef->btnCancel, SIGNAL(clicked()), this, SLOT(unitListBefShow()));
    connect(unitListAft->btnCancel, SIGNAL(clicked()), this, SLOT(unitListAftShow()));

    this->unitListBef->hide();
    this->unitListAft->hide();

    m_picturetowhite = new PictureToWhite();

    rateMap = {
        {"Algerian dinar",      "DZD"},
        {"Australian dollar",   "AUD"},
        {"Bahrain Dinar",       "BHD"},
        {"Bolivar Fuerte",      "VEF"},
        {"Botswana pula",       "BWP"},
        {"Brazilian real",      "BRL"},
        {"Brunei dollar",       "BND"},
        {"Canadian dollar",     "CAD"},
        {"Chilean peso",        "CLP"},
        {"Chinese yuan",        "CNY"},
        {"Colombian peso",      "COP"},
        {"Czech koruna",        "CZK"},
        {"Danish krone",        "DKK"},
        {"Euro",                "EUR"},
        {"Hungarian Forint",    "HUF"},
        {"Icelandic Krona",     "ISK"},
        {"Indian rupee",        "INR"},
        {"Indonesian Rupiah",   "IDR"},
        {"Iranian Rial",        "IRR"},
        {"Israeli New Shekel",  "ILS"},
        {"Japanese yen",        "JPY"},
        {"Kazakhstani Tenge",   "KZT"},
        {"Korean won",          "KRW"},
        {"Kuwaiti dinar",       "KWD"},
        {"Libyan Dinar",        "LYD"},
        {"Malaysian ringgit",   "MYR"},
        {"Mauritian rupee",     "MUR"},
        {"Mexican peso",        "MXN"},
        {"Nepalese Rupee",      "NPR"},
        {"New Zealand dollar",  "NZD"},
        {"Norwegian krone",     "NOK"},
        {"Nuevo Sol",           "PEN"},
        {"Omani rial",          "OMR"},
        {"Pakistani Rupee",     "PKR"},
        {"Peruvian sol",        "PEN"},
        {"Philippine peso",     "PHP"},
        {"Polish zloty",        "PLN"},
        {"Qatari riyal",        "QAR"},
        {"Rial Omani",          "OMR"},
        {"Russian ruble",       "RUB"},
        {"Saudi Arabian riyal", "SAR"},
        {"Singapore dollar",    "SGD"},
        {"South African rand",  "ZAR"},
        {"Sri Lanka Rupee",     "LKR"},
        {"Swedish krona",       "SEK"},
        {"Swiss franc",         "CHF"},
        {"Thai baht",           "THB"},
        {"Trinidadian dollar",  "TTD"},
        {"Tunisian Dinar",      "TND"},
        {"U.A.E. dirham",       "AED"},
        {"U.K. pound",          "GBP"},
        {"Uruguayan peso",      "UYU"},
        {"U.S. dollar",         "USD"},
        {NULL, NULL}
    };

    // 没有ARS
    currencyInfoUS = {
        {"AED", "UAE Dirham"},
        {"ARS", "Argentinian peso"},
        {"AUD", "Australian Dollar"},
        {"BGN", "Bulgarian Lev"},
        {"BHD", "Bahraini Dinar"},
        {"BND", "Brunei Dollar"},
        {"BRL", "Brazilian Real"},
        {"BSD", "Bahaman Dollar"},
        {"BWP", "Botswana Pula"},
        {"CAD", "Canadian Dollar"},
        {"CFA", "CFA Franc"},
        {"CHF", "Swiss Franc"},
        {"CLP", "Chilean Peso"},
        {"CNY", "Chinese Yuan"},
        {"COP", "Colombian Peso"},
        {"CZK", "Czech Koruna"},
        {"DKK", "Danish Krone"},
        {"DOP", "Dominican peso"},
        {"DZD", "Algerian Dinar"},
        {"EEK", "Estonian Kroon"},
        {"EGP", "Egyptian pound"},
        {"EUR", "Euro"},
        {"FJD", "Fijian dollar"},
        {"GBP", "Pound Sterling"},
        {"GTQ", "Guatemalan Quetzal"},
        {"HKD", "Hong Kong Dollar"},
        {"HRK", "Croatian Kuna"},
        {"HUF", "Hungarian Forint"},
        {"IDR", "Indonesian Rupiah"},
        {"ILS", "Israeli New Shekel"},
        {"INR", "Indian Rupee"},
        {"IRR", "Iranian Rial"},
        {"ISK", "Icelandic Krona"},
        {"JPY", "Japanese Yen"},
        {"KRW", "South Korean Won"},
        {"KWD", "Kuwaiti Dinar"},
        {"KZT", "Kazakhstani Tenge"},
        {"LKR", "Sri Lankan Rupee"},
        {"LTL", "Lithuanian Litas"},
        {"LVL", "Latvian Lats"},
        {"LYD", "Libyan Dinar"},
        {"MUR", "Mauritian Rupee"},
        {"MVR", "Maldivian Rupee"},
        {"MXN", "Mexican Peso"},
        {"MYR", "Malaysian Ringgit"},
        {"NOK", "Norwegian Krone"},
        {"NPR", "Nepalese Rupee"},
        {"NZD", "New Zealand Dollar"},
        {"OMR", "Omani Rial"},
        {"PAB", "Panamanian balbos"},
        {"PEN", "Peruvian Nuevo Sol"},
        {"PHP", "Philippine Peso"},
        {"PKR", "Pakistani Rupee"},
        {"PLN", "Polish Zloty"},
        {"PYG", "Paraguayan Guaran"},
        {"QAR", "Qatari Riyal"},
        {"RON", "New Romanian Leu"},
        {"RUB", "Russian Rouble"},
        {"SAR", "Saudi Riyal"},
        {"SEK", "Swedish Krona"},
        {"SGD", "Singapore Dollar"},
        {"THB", "Thai Baht"},
        {"TND", "Tunisian Dinar"},
        {"TRY", "New Turkish Lira"},
        {"TTD", "T&T Dollar (TTD)"},
        {"TWD", "Taiwan Dollar"},
        {"UAH", "Ukrainian Hryvnia"},
        {"USD", "US Dollar"},
        {"UYU", "Uruguayan Peso"},
        {"VEF", "Venezuelan Bolívar"},
        {"ZAR", "South African Rand"},
        {NULL, NULL}
    };

    // 新增ARS、BSD、DOP、EGP、FJD、GTQ、MVR、PAB、PYG、TWD、UAH
    currencyInfoCN = {
        {"AED", "阿联酋迪拉姆"},
        {"ARS", "阿根廷比索"},
        {"AUD", "澳大利亚元"},
        {"BGN", "保加利亚列弗"},
        {"BHD", "巴林第纳尔"},
        {"BND", "文莱元"},
        {"BRL", "巴西雷亚尔"},
        {"BSD", "巴哈马元"},
        {"BWP", "博茨瓦纳普拉"},
        {"CAD", "加拿大元"},
        {"CFA", "中非法郎"},
        {"CHF", "瑞士法郎"},
        {"CLP", "智利比索"},
        {"CNY", "人民币"},
        {"COP", "哥伦比亚比索"},
        {"CZK", "捷克克朗"},
        {"DKK", "丹麦克朗"},
        {"DOP", "多米尼加比索"},
        {"DZD", "阿尔及利亚第纳尔"},
        {"EEK", "爱沙尼亚克朗"},
        {"EGP", "埃及镑"},
        {"EUR", "欧元"},
        {"FJD", "斐济元"},
        {"GBP", "英镑"},
        {"GTQ", "危地马拉格查尔"},
        {"HKD", "港币"},
        {"HRK", "克罗地亚库纳"},
        {"HUF", "匈牙利福林"},
        {"IDR", "印度尼西亚卢比"},
        {"ILS", "以色列新谢克尔"},
        {"INR", "印度卢比"},
        {"IRR", "伊朗里亚尔"},
        {"ISK", "冰岛克朗"},
        {"JPY", "日元"},
        {"KRW", "韩元"},
        {"KWD", "科威特第纳尔"},
        {"KZT", "哈萨克斯坦腾格"},
        {"LKR", "斯里兰卡卢比"},
        {"LTL", "立陶宛立特"},
        {"LVL", "拉脱维亚拉特"},
        {"LYD", "利比亚第纳尔"},
        {"MUR", "毛里求斯卢比"},
        {"MVR", "马尔代夫卢比"},
        {"MXN", "墨西哥比索"},
        {"MYR", "马来西亚林吉特"},
        {"NOK", "挪威克朗"},
        {"NPR", "尼泊尔卢比"},
        {"NZD", "新西兰元"},
        {"OMR", "阿曼里亚尔"},
        {"PAB", "巴拿马巴波亚"},
        {"PEN", "秘鲁新索尔"},
        {"PHP", "菲律宾比索"},
        {"PKR", "巴基斯坦卢比"},
        {"PLN", "波兰兹罗提"},
        {"PYG", "巴拉圭瓜拉尼"},
        {"QAR", "卡塔尔里亚尔"},
        {"RON", "新罗马尼亚列伊"},
        {"RUB", "俄罗斯卢布"},
        {"SAR", "沙特里亚尔"},
        {"SEK", "瑞典克朗"},
        {"SGD", "新加坡元"},
        {"THB", "泰铢"},
        {"TND", "突尼斯第纳尔"},
        {"TRY", "新土尔其里拉"},
        {"TTD", "特立尼达和多巴哥元"},
        {"TWD", "台币"},
        {"UAH", "乌克兰格里夫纳"},
        {"USD", "美元"},
        {"UYU", "乌拉圭比索"},
        {"VEF", "委内瑞拉玻利瓦尔"},
        {"ZAR", "南非兰特"},
        {NULL, NULL}
    };

    m_firstLoad = true;

    defUpdateRate();

    updateRate();

    this->setAutoFillBackground(true);
    this->setBackgroundRole(QPalette::Base);

    toolUnitBefWid->setProperty("isWindowButton", 0x1);
    toolUnitBefWid->setProperty("useIconHighlightEffect", 0x2);

    toolUnitAftWid->setProperty("isWindowButton", 0x1);
    toolUnitAftWid->setProperty("useIconHighlightEffect", 0x2);
}

// 设置组件样式
void ToolModelOutput::setWidgetStyle(bool resetFontSize)
{
    if (WidgetStyle::themeColor == 0) {
        toolLabUpdate->setStyleSheet("font-size:16px;color:#272A2D;");

        toolLabTime->setStyleSheet("font-size:10px;color:#272A2D;");
        toolLabRate->setStyleSheet("font-size:10px;color:#272A2D;");
        toolUpdateWid->setStyleSheet("background:#F3F3F3;opacity:1;");

        QString unitLabStyle  = "font-family:SourceHanSansCN-ExtraLight;font-size:18px;color:#272A2D;line-height:22px;";
        toolLabUnitBef->setStyleSheet(unitLabStyle);
        toolLabIconBef->setPixmap(QIcon::fromTheme("ukui-down-symbolic").pixmap(16, 16));
        //toolLabIconBef->setStyleSheet("border-image:url(:/image/downward.png);");
        toolUnitBefWid->setStyleSheet("QPushButton{background:#F3F3F3;color:#272A2D;opacity:1;}"
                                      "QPushButton:hover{background-color:#F3F3F3;}");
        toolLabUnitAft->setStyleSheet(unitLabStyle);
        toolLabIconAft->setPixmap(QIcon::fromTheme("ukui-down-symbolic").pixmap(16, 16));
        //toolLabIconAft->setStyleSheet("border-image:url(:/image/downward.png);");
        toolUnitAftWid->setStyleSheet("QPushButton{background:#F3F3F3;color:#272A2D;opacity:1;}"
                                      "QPushButton:hover{background-color:#F3F3F3;}");
        // 数据输出界面
        this->toolLabHis->setStyleSheet("color:#8C8C8C;font-size:12px;font-weight:15px;line-height:50px;");
        this->toolLabAft->setStyleSheet("color:#272A2D;font-size:40px;font-weight:15px;line-height:50px;");

        if (resetFontSize) {
            this->toolLabBef->setStyleSheet("color:#272A2D;font-size:48px;font-weight:15px;");
        } else {
            QString fontSizeStr = QString::number(toolLabBef->font().pixelSize());
            this->toolLabBef->setStyleSheet("color:#272A2D;font-size:"+ fontSizeStr +"px;font-weight:15px;");
        }

        // this->setStyleSheet("background-color:#8C8C8C;border-radius:4px;");

    } else if (WidgetStyle::themeColor == 1) {
        toolLabUpdate->setStyleSheet("font-family:SourceHanSansCN-ExtraLight;font-size:16px;color:#FFFFFF;");

        toolLabTime->setStyleSheet("font-family:NotoSansHans-Light;font-size:10px;color:#A6A6A6;line-height:17px;");
        toolLabRate->setStyleSheet("font-family:NotoSansHans-Light;font-size:10px;color:#A6A6A6;line-height:17px;");
        toolUpdateWid->setStyleSheet("background:#18181A;opacity:0.75;");

        QString unitLabStyle  = "font-family:SourceHanSansCN-ExtraLight;font-size:18px;color:#FFFFFF;line-height:22px;";
        toolLabUnitBef->setStyleSheet(unitLabStyle);
        toolLabIconBef->setPixmap(m_picturetowhite->drawSymbolicColoredPixmap(QIcon::fromTheme("ukui-down-symbolic").pixmap(16, 16)));
        //toolLabIconBef->setStyleSheet("border-image:url(:/image/downward.png);");
        toolUnitBefWid->setStyleSheet("QPushButton{background:#18181A;color:#FFFFFF;opacity:0.75;}"
                                      "QPushButton:hover{background-color:#18181A;}");
        toolLabUnitAft->setStyleSheet(unitLabStyle);
        toolLabIconAft->setPixmap(m_picturetowhite->drawSymbolicColoredPixmap(QIcon::fromTheme("ukui-down-symbolic").pixmap(16, 16)));
        //toolLabIconAft->setStyleSheet("border-image:url(:/image/downward.png);");
        toolUnitAftWid->setStyleSheet("QPushButton{background:#18181A;color:#FFFFFF;opacity:0.75;}"
                                      "QPushButton:hover{background-color:#18181A;}");

        // 数据输出界面
        this->toolLabHis->setStyleSheet("color:#8C8C8C;font-size:12px;font-weight:15px;line-height:50px;");
        this->toolLabAft->setStyleSheet("color:#FFFFFF;font-size:40px;font-weight:15px;line-height:50px;");

        if (resetFontSize) {
            this->toolLabBef->setStyleSheet("color:#FFFFFF;font-size:48px;font-weight:15px;");
        }
        else {
            QString fontSizeStr = QString::number(toolLabBef->font().pixelSize());
            this->toolLabBef->setStyleSheet("color:#FFFFFF;font-size:"+ fontSizeStr +"px;font-weight:15px;");
        }
    }


    QListWidgetItem *item = unitListBef->getItem();
    unitListBef->setSelectItem(unitListBef->unitList->row(item));
    item = unitListAft->getItem();
    unitListAft->setSelectItem(unitListAft->unitList->row(item));
}

// 初始化单位列表
void ToolModelOutput::initUnitList(QString listStr)
{
    QStringList unitStrList = listStr.split(",");

    unitListBef->unitList->clear();
    unitListAft->unitList->clear();

    QListWidgetItem *unitBeforeItem[unitStrList.size()];
    QListWidgetItem *unitAfterItem[unitStrList.size()];

    for (int i = 0; i < unitStrList.size(); i++) {

        // 获取汇率名称
        QMap<QString, QString>::iterator it;
        QString itemName;

        // 筛选需要显示的货币
        QString locale = QLocale::system().name();
        if (locale == "zh_CN") {
            if (currencyInfoCN.contains(unitStrList[i])) {

                it = currencyInfoCN.find(unitStrList[i]);
                if (it != currencyInfoCN.end()) {
                    itemName = it.value();
                }
                else {
                    continue ;
                }
            } else {
                continue;
            }
        }
        else if (locale == "en_US") {
            if (currencyInfoUS.contains(unitStrList[i])) {

                it = currencyInfoUS.find(unitStrList[i]);
                if (it != currencyInfoUS.end()) {
                    itemName = it.value();
                }
                else {
                    continue ;
                }
            } else {
                continue;
            }
        }

        UnitListItem *listItemBef = new UnitListItem(unitListBef->unitList);
        UnitListItem *listItemAft = new UnitListItem(unitListAft->unitList);
        
        // 设置列表项的单位名称和单位符号
        listItemBef->unitItemName->setText(itemName);
        listItemAft->unitItemName->setText(itemName);

        listItemBef->unitItemSymb->setText(unitStrList[i]);
        listItemAft->unitItemSymb->setText(unitStrList[i]);

        unitBeforeItem[i] = new QListWidgetItem(unitListBef->unitList, i);
        unitAfterItem[i]  = new QListWidgetItem(unitListAft->unitList, i);

        unitListBef->unitList->setItemWidget(unitBeforeItem[i], listItemBef);
        unitListAft->unitList->setItemWidget(unitAfterItem[i],  listItemAft);
    }

    unitListBef->unitList->setSpacing(2);
    unitListAft->unitList->setSpacing(2);

    // 默认汇率选择人民币和美元
    unitListBef->unitList->setCurrentRow(12);
    unitListAft->unitList->setCurrentRow(63);
    unitListBef->setSelectItem(12);
    unitListAft->setSelectItem(63);

    return;
}

/* 将正常计算结果 乘以汇率 进行转换 */
void ToolModelOutput::unitConversion()
{
    QString strBefore = this->toolLabBef->text().remove(QRegExp(","));
    strBefore.replace("×" , "*");
    strBefore.replace("÷" , "/");
    strBefore.replace(SUB , "-");

    /* 判断获取的表达式是否正常 */
    /* 之前的计算已经进行判断 , 此处判断表达式中含有数字 则认为正常 , 目的在与 排除 inf Error 等表示错误的表达式 */
    bool flag = false;
    std::string stdStr = strBefore.toStdString();
    const char *cStr = stdStr.c_str();
    while(*cStr != '\0') {
        if (*cStr >= '0' && *cStr <= '9') {
            flag = true;
            break;
        }
        cStr++;
    }

    if (!flag) {
        return;
    }

    /* 乘以汇率 */
    double douResult = calculator(calculator(strBefore) + "*" + QString::number(toolDouRate)).toDouble();

    QString strResult = QString::number(douResult);

    if (strResult.contains("inf")) {
        strResult = tr("Error!");
    }

    this->toolLabAft->setText(strResult);

    return;
}

// 单位换算历史记录转换
QString ToolModelOutput::unitConvHistory(QString hisText)
{
    QString resHisText;

    hisText.replace("×", "*");
    hisText.replace("÷", "/");
    hisText.replace(SUB, "-");

    /* 默认使用人民币/美元汇率 */
    if (toolDouRate == 0) {
        toolDouRate = 0.15;
    }

    QStringList singerHisList = (hisText).split("=");
    QString dataBef = singerHisList[singerHisList.size()-1].remove(QRegExp(",")).remove("\n");
    double douResult = calculator( calculator (dataBef) + "*" + QString::number(toolDouRate)   ).toDouble();
    QString newHisText = dataBef + " " + toolRateSymbBef + " = " + QString::number(douResult) + " " + toolRateSymbAft + "\n";

    toolDisHistory.append(newHisText);

    // 获取历史记录
    int size = toolDisHistory.size();
    int hisIndex = (size - 4 >= 0) ? (size - 4) : 0;
    for (int i = hisIndex; i < size; i++) {
        resHisText = resHisText + toolDisHistory.at(i);
    }

    // 去除末尾换行符
    resHisText.chop(1);

    return resHisText;
}

// 汇率更新函数
void ToolModelOutput::updateRate()
{
    // 暂时隐藏
//    toolIconUpdate->setEnabled(false);

//    updateRateThread = new UpdateRateThread();

//    connect(updateRateThread, &UpdateRateThread::isDone, this, &ToolModelOutput::dealDone);

//    updateRateThread->start();
    // 更新界面的数据 时间 汇率
    QDateTime date = QDateTime::currentDateTime();
    QString strTime = date.toString("yyyy.MM.dd hh:mm");

    toolLabTime->setText(strTime);

}

void ToolModelOutput::defUpdateRate()
{
    if (m_firstLoad) {
        m_firstLoad = false;

        QString strRateAll = "{\"base\":\"CNY\",\"date\":\"2021-06-11\",\"time_last_updated\":1623369601,\"rates\":{\"AED\":0.575,\"AFN\":12.34,\"ALL\":15.77,\"AMD\":81.17,\"ANG\":0.28,\"AOA\":101.21,\"ARS\":14.86,\"AUD\":0.202,\"AWG\":0.28,\"AZN\":0.266,\"BAM\":0.251,\"BBD\":0.313,\"BDT\":13.25,\"BGN\":0.251,\"BHD\":0.0588,\"BIF\":308.3,\"BMD\":0.156,\"BND\":0.207,\"BOB\":1.08,\"BRL\":0.791,\"BSD\":0.156,\"BTN\":11.42,\"BWP\":1.66,\"BYN\":0.39,\"BZD\":0.313,\"CAD\":0.189,\"CDF\":312.11,\"CHF\":0.14,\"CLP\":112.58,\"CNY\":1,\"COP\":561.66,\"CRC\":96.53,\"CUC\":0.156,\"CUP\":4.03,\"CVE\":14.17,\"CZK\":3.27,\"DJF\":27.81,\"DKK\":0.959,\"DOP\":8.91,\"DZD\":20.95,\"EGP\":2.45,\"ERN\":2.35,\"ETB\":6.78,\"EUR\":0.129,\"FJD\":0.319,\"FKP\":0.111,\"FOK\":0.959,\"GBP\":0.111,\"GEL\":0.493,\"GGP\":0.111,\"GHS\":0.91,\"GIP\":0.111,\"GMD\":8.11,\"GNF\":1537.96,\"GTQ\":1.21,\"GYD\":32.8,\"HKD\":1.21,\"HNL\":3.76,\"HRK\":0.968,\"HTG\":14.3,\"HUF\":44.62,\"IDR\":2233.07,\"ILS\":0.51,\"IMP\":0.111,\"INR\":11.42,\"IQD\":228.91,\"IRR\":6534.46,\"ISK\":18.91,\"JMD\":23.21,\"JOD\":0.111,\"JPY\":17.14,\"KES\":16.89,\"KGS\":13.2,\"KHR\":640.25,\"KID\":0.202,\"KMF\":63.23,\"KRW\":174.61,\"KWD\":0.0469,\"KYD\":0.13,\"KZT\":66.88,\"LAK\":1474.08,\"LBP\":235.9,\"LKR\":31,\"LRD\":26.8,\"LSL\":2.13,\"LYD\":0.699,\"MAD\":1.38,\"MDL\":2.76,\"MGA\":593,\"MKD\":7.92,\"MMK\":257.31,\"MNT\":448.17,\"MOP\":1.25,\"MRU\":5.67,\"MUR\":6.36,\"MVR\":2.41,\"MWK\":125.78,\"MXN\":3.08,\"MYR\":0.644,\"MZN\":9.9,\"NAD\":2.13,\"NGN\":66.76,\"NIO\":5.46,\"NOK\":1.3,\"NPR\":18.28,\"NZD\":0.218,\"OMR\":0.0602,\"PAB\":0.156,\"PEN\":0.603,\"PGK\":0.549,\"PHP\":7.47,\"PKR\":24.34,\"PLN\":0.576,\"PYG\":1041.92,\"QAR\":0.57,\"RON\":0.633,\"RSD\":15.11,\"RUB\":11.28,\"RWF\":156.52,\"SAR\":0.587,\"SBD\":1.24,\"SCR\":2.57,\"SDG\":67.49,\"SEK\":1.29,\"SGD\":0.207,\"SHP\":0.111,\"SLL\":1606.89,\"SOS\":90.77,\"SRD\":2.21,\"SSP\":27.82,\"STN\":3.15,\"SYP\":197.22,\"SZL\":2.13,\"THB\":4.88,\"TJS\":1.77,\"TMT\":0.547,\"TND\":0.427,\"TOP\":0.346,\"TRY\":1.33,\"TTD\":1.06,\"TVD\":0.202,\"TWD\":4.33,\"TZS\":363.16,\"UAH\":4.23,\"UGX\":550.96,\"USD\":0.156,\"UYU\":6.81,\"UZS\":1656.53,\"VES\":490206.01,\"VND\":3583.66,\"VUV\":16.88,\"WST\":0.393,\"XAF\":84.3,\"XCD\":0.422,\"XDR\":0.108,\"XOF\":84.3,\"XPF\":15.34,\"YER\":39.08,\"ZAR\":2.13,\"ZMW\":3.53}}";

        // 格式化处理汇率信息字符串键值对
        strRateList = strRateAll.split(QRegExp("[{} :,\"\n]"));
        strRateList.removeAll("");

        for (int i = 0 ; i < 7 ; i++) {
            strRateList.removeAt(0);
        }

        // 获取汇率单位标识
        QString strRateKey = strRateList.at(0);
        for (int i = 2; i < strRateList.size(); i++) {
            if (i % 2 == 0) {
                strRateKey.append(",");
                strRateKey.append(strRateList.at(i));
            }
        }

        // 初始化选择列表的汇率选项
        initUnitList(strRateKey);

    }

    // 更新界面的数据 时间 汇率
    QDateTime date = QDateTime::currentDateTime();
    QString strTime = date.toString("yyyy.MM.dd hh:mm");

    QString rateNameBef = toolRateSymbBef;
    QString rateNameAft = toolRateSymbAft;

    QString labelRate = "1 "+ rateNameBef + " = " +
                        QString::number(toolDouRate,'f',2) + " " + rateNameAft;
    toolLabTime->setText(strTime);
    toolLabRate->setText(labelRate);

    updateRateName();
}

/* 控制换算前的单位列表 */
void ToolModelOutput::unitListBefShow()
{
    if (unitListBef->isHidden()) {
        int b_x = toolUnitBefWid->x() + toolUnitBefWid->width();
        int b_y = toolUnitBefWid->y();
        unitListBef->setGeometry(QRect(b_x, b_y, 280, 410));

        unitListBef->show();
        unitListBef->raise();
        unitListBef->setFocus();

        toolUnitBefWid->setBackgroundRole(QPalette::Window);
    }
    else {
        unitListBef->hide();
        toolUnitBefWid->setBackgroundRole(QPalette::Base);
    }
}

/* 控制换算后的单位列表 */
void ToolModelOutput::unitListAftShow()
{
    if (unitListAft->isHidden()) {
        int a_x = toolUnitAftWid->x() + toolUnitAftWid->width();
        int a_y = toolUnitAftWid->y();
        unitListAft->setGeometry(QRect(a_x , a_y , 280 , 410));
        unitListAft->show();
        unitListAft->raise();
        unitListAft->setFocus();
    }
    else {
        unitListAft->hide();
    }
}

// 选择换算前的单位列表
void ToolModelOutput::unitListBefSelect(QListWidgetItem *item)
{

    unitListBef->hide();
}

// 选择换算后的单位列表
void ToolModelOutput::unitListAftSelect(QListWidgetItem *item)
{
    QString rateSymbol = item->text();
    QString rateName   = rateSymbol;

    // 更新汇率比率显示
    QString labelRate = toolLabRate->text();
    labelRate = labelRate.section("=", 0, 0) + "= " + QString::number(toolDouRate) + " " + rateName;
    toolLabRate->setText(labelRate);

    toolLabUnitAft->setText(rateSymbol + "\n" + rateName);

    unitListAft->hide();
}

/* 选择货币单位列表 */
void ToolModelOutput::unitListSelect(QListWidgetItem *item)
{
    if (unitListAft->isHidden()) {
        // 获取QWidget
        QWidget* itemWid = unitListBef->unitList->itemWidget(item);
        // 获取所有的QLabel
        QList<QLabel*> labelList = itemWid->findChildren<QLabel*>();

        unitListBef->setSelectItem(unitListBef->unitList->row(item));

        toolRateSymbBef = labelList[1]->text();
        updateRateName();
        unitListBef->hide();
    } else {
        // 获取QWidget
        QWidget* itemWid = unitListAft->unitList->itemWidget(item);
        // 获取所有的QLabel
        QList<QLabel*> labelList = itemWid->findChildren<QLabel*>();

        unitListAft->setSelectItem(unitListAft->unitList->row(item));

        toolRateSymbAft = labelList[1]->text();
        updateRateName();
        unitListAft->hide();
    }
}

void ToolModelOutput::dealDone(QStringList list)
{
    if(list.isEmpty()){
        return;
    }
    strRateList = list;
    // 获取汇率单位标识
    QString strRateKey = strRateList.at(0);
    for (int i = 2; i < strRateList.size(); i++) {
        if (i % 2 == 0) {
            strRateKey.append(",");
            strRateKey.append(strRateList.at(i));
        }
    }

    // 初始化选择列表的汇率选项
    initUnitList(strRateKey);

    // 更新界面的数据 时间 汇率
    QDateTime date = QDateTime::currentDateTime();
    QString strTime = date.toString("yyyy.MM.dd hh:mm");


    QString rateNameBef = toolRateSymbBef;
    QString rateNameAft = toolRateSymbAft;

    QString labelRate = "1 "+ rateNameBef + " = " +
                        QString::number(toolDouRate,'f',2) + " " + rateNameAft;
    toolLabTime->setText(strTime);
    toolLabRate->setText(labelRate);

    updateRateName();

    toolIconUpdate->setEnabled(true);

    //停止线程
    updateRateThread->quit();

    //等待线程处理完手头工作
    updateRateThread->wait();
}

// 根据符号更新货币名称并显示
void ToolModelOutput::updateRateName()
{
    /* 获取汇率索引值 */
    int idxRateBef = strRateList.indexOf(toolRateSymbBef) + 1;
    int idxRateAft = strRateList.indexOf(toolRateSymbAft) + 1;

    /* 获取汇率比率 */
    toolDouRate = (strRateList[idxRateAft]).toDouble() / (strRateList[idxRateBef]).toDouble();

    /* 获取汇率名称 */
    QMap<QString, QString>::iterator it;

    QString locale = QLocale::system().name();

    if (locale == "zh_CN") {
        it = currencyInfoCN.find(toolRateSymbBef);
        if (it != currencyInfoCN.end()) {
            toolRateNameBef = it.value();
        }

        it = currencyInfoCN.find(toolRateSymbAft);
        if (it != currencyInfoCN.end()) {
            toolRateNameAft = it.value();
        }
    }
    else if (locale == "en_US") {
        it = currencyInfoUS.find(toolRateSymbBef);
        if (it != currencyInfoUS.end()) {
            toolRateNameBef = it.value();
        }

        it = currencyInfoUS.find(toolRateSymbAft);
        if (it != currencyInfoUS.end()) {
            toolRateNameAft = it.value();
        }
    }

    // 更新汇率比率显示
    QString labelRate = toolLabRate->text();
    labelRate = "1 " + toolRateSymbBef + " = " + QString::number(toolDouRate, 'f', 2) + " " + toolRateSymbAft;
    toolLabRate->setText(labelRate);

    // 更新汇率名称和符号显示
    toolLabUnitBef->setText(toolRateNameBef + "\n" + toolRateSymbBef);
    toolLabUnitAft->setText(toolRateNameAft + "\n" + toolRateSymbAft);

    QFont font(toolLabUnitBef->font());
    font.setPixelSize(18);  //QFont像素大小设置为18，然后再次设置到QLabel中
    toolLabUnitBef->setFont(font);
    toolLabUnitAft->setFont(font);

    QFontMetrics fontmts = toolLabUnitBef->fontMetrics();
    int dif = fontmts.width(toolRateNameBef) - toolLabUnitBef->width();
    if (dif > 0) {
        QString str = fontmts.elidedText(toolRateNameBef,Qt::ElideRight,toolLabUnitBef->width());
        toolLabUnitBef->setText(str + "\n" + toolRateSymbBef);
        toolLabUnitBef->setToolTip(toolRateNameBef);
    }

    fontmts = toolLabUnitAft->fontMetrics();
    dif = fontmts.width(toolRateNameAft) - toolLabUnitAft->width();
    if (dif > 0) {
        QString str = fontmts.elidedText(toolRateNameAft,Qt::ElideRight,toolLabUnitAft->width());
        toolLabUnitAft->setText(str + "\n" + toolRateSymbAft);
        toolLabUnitAft->setToolTip(toolRateNameAft);
    }

    // 更新汇率换算结果
    unitConversion();
}

// 换算器按钮界面
ToolModelButton::ToolModelButton(QWidget *parent)
    : QWidget(parent)
{
    // 初始化组件
    this->setWidgetUi();

    // 设置组件样式
    this->setWidgetStyle();
}

// 初始化组件
void ToolModelButton::setWidgetUi()
{
    for (int i = 0; i < 10; i++) {
        btnNum[i] = new BasicButton(this);
        btnNum[i]->setText(QString::number(i));
        btnNum[i]->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
        btnNum[i]->setIconSize(QSize(106, 62));
        btnNum[i]->setProperty("useButtonPalette", true);
    }

    btnClear = new BasicButton(this);
    btnPer = new BasicButton(this);
    btnDelete = new BasicButton(this);

    btnDiv = new BasicButton(this);
    btnMulti = new BasicButton(this);
    btnSub = new BasicButton(this);
    btnAdd = new BasicButton(this);
    btnEqual = new BasicButton(this);

    btnPoint = new BasicButton(this);

    m_picturetowhite = new PictureToWhite();

    // 设置按钮的显示文本
    QString btnList = "C,÷,×,B,ｰ,+,=,%,.";
    QStringList btnNameList= btnList.split(",");
    int index = 0;

    btnClear->setText(btnNameList[index++]);
    btnDiv->setText(btnNameList[index++]);
    btnMulti->setText(btnNameList[index++]);
    btnDelete->setText(btnNameList[index++]);
    btnSub->setText(btnNameList[index++]);
    btnAdd->setText(btnNameList[index++]);
    btnEqual->setText(btnNameList[index++]);
    btnPer->setText(btnNameList[index++]);
    btnPoint->setText(btnNameList[index++]);

    for (int i = 0; i < 10; i++) {
        btnNum[i]->setIcon(QIcon(":/image/newIcon/standard/btnNum"+ QString::number(i) +".svg"));
    }

    btnClear->setIcon(QIcon(":/image/newIcon/standard/btnClear.svg"));
    btnDiv->setIcon(QIcon(":/image/newIcon/standard/btnDiv.svg"));
    btnMulti->setIcon(QIcon(":/image/newIcon/standard/btnMulti.svg"));
    btnDelete->setIcon(QIcon(":/image/newIcon/standard/btnDelete.svg"));
    btnSub->setIcon(QIcon(":/image/newIcon/standard/btnSub.svg"));
    btnAdd->setIcon(QIcon(":/image/newIcon/standard/btnAdd.svg"));
    btnEqual->setIcon(QIcon(":/image/newIcon/standard/btnEqual.svg"));
    btnPer->setIcon(QIcon(":/image/newIcon/standard/btnPer.svg"));
    btnPoint->setIcon(QIcon(":/image/newIcon/standard/btnPoint.svg"));

    btnClear->setIconSize(QSize(106, 62));
    btnDiv->setIconSize(QSize(106, 62));
    btnMulti->setIconSize(QSize(106, 62));
    btnDelete->setIconSize(QSize(85,  50));
    btnSub->setIconSize(QSize(106, 62));
    btnAdd->setIconSize(QSize(106, 62));
    btnEqual->setIconSize(QSize(106, 62));
    btnPer->setIconSize(QSize(106, 62));
    btnPoint->setIconSize(QSize(106, 62));

    // 设置按钮自适应放缩
    btnClear->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    btnDiv->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    btnMulti->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    btnDelete->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    btnSub->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    btnAdd->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    btnEqual->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    btnPer->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    btnPoint->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);

    btnClear->setProperty("useButtonPalette", true);
    btnDiv->setProperty("useButtonPalette", true);
    btnMulti->setProperty("useButtonPalette", true);
    btnDelete->setProperty("useButtonPalette", true);
    btnSub->setProperty("useButtonPalette", true);
    btnAdd->setProperty("useButtonPalette", true);
    btnEqual->setProperty("useButtonPalette", true);
    btnPer->setProperty("useButtonPalette", true);
    btnPoint->setProperty("useButtonPalette", true);

    btnEqual->setProperty("isImportant", true);

    // 将按钮进行网格布局
    QGridLayout *btnLayout = new QGridLayout();
    btnLayout->addWidget(btnClear, 0, 0, 1, 1);
    btnLayout->addWidget(btnPer, 0, 1, 1, 1);
    btnLayout->addWidget(btnDelete, 0, 2, 1, 1);
    btnLayout->addWidget(btnDiv, 0, 3, 1, 1);
    btnLayout->addWidget(btnMulti, 1, 3, 1, 1);
    btnLayout->addWidget(btnSub, 2, 3, 1, 1);
    btnLayout->addWidget(btnAdd, 3, 3, 1, 1);
    btnLayout->addWidget(btnPoint, 4, 2, 1, 1);
    btnLayout->addWidget(btnEqual, 4, 3, 1, 1);

    // 数字按钮布局
    // btnLayout->addWidget(btnDZero, 4, 0, 1, 1);
    btnLayout->addWidget(btnNum[0], 4, 0, 1, 2);
    for (int i = 1; i < 10; i++) {
        btnLayout->addWidget(btnNum[i], 3 - (i - 1)/3, (i - 1) % 3, 1, 1);
    }

    // 设置间距和背景样式
    btnLayout->setSpacing(2);
    btnLayout->setMargin(1);

    //this->setFixedHeight(320);
    this->setLayout(btnLayout);

}

// 设置组件样式
void ToolModelButton::setWidgetStyle()
{
//    QString btnStyle;
//    QString btnOpStyle;
//    QString btnEqualStyle;

//    if (WidgetStyle::themeColor == 0) {
//        for (int i = 0; i < 10; i++) {
//            btnNum[i]->setIcon(QIcon(":/image/newIcon/standard/btnNum"+ QString::number(i) +".svg"));
//        }

//        // btnDZero->setIcon(QIcon(":/image/newIcon/standard/btnNum00.svg"));

//        btnClear->setIcon(QIcon(":/image/newIcon/standard/btnClear.svg"));
//        btnDiv->setIcon(QIcon(":/image/newIcon/standard/btnDiv.svg"));
//        btnMulti->setIcon(QIcon(":/image/newIcon/standard/btnMulti.svg"));
//        btnDelete->setIcon(QIcon(":/image/newIcon/standard/btnDelete.svg"));
//        btnSub->setIcon(QIcon(":/image/newIcon/standard/btnSub.svg"));
//        btnAdd->setIcon(QIcon(":/image/newIcon/standard/btnAdd.svg"));
//        btnEqual->setIcon(QIcon(":/image/newIcon/standard/btnEqual.svg"));
//        btnPer->setIcon(QIcon(":/image/newIcon/standard/btnPer.svg"));
//        btnPoint->setIcon(QIcon(":/image/newIcon/standard/btnPoint.svg"));


//        btnStyle = "QPushButton{"
//                   "background-color:#F3F3F3;border-radius:4px;"
//                   "font-size:42px;font-family:HelveticaNeueLTPro-UltLt;color:#FFFFFF;"
//                   "}"
//                   "QPushButton:hover{background-color:#E8E8E8;}";

//        btnOpStyle = "QPushButton{"
//                     "background-color:#F5F5F8;border-radius:4px;"
//                     "font-size:35px;font-family:HelveticaNeueLTPro-UltLt;color:#FFFFFF;"
//                     "}"
//                     "QPushButton:hover{background-color:#E8E8E8;}";

//        btnEqualStyle = "QPushButton{"
//                        "background-color:#E64A19;border-radius:4px;"
//                        "font-size:48px;font-family:HelveticaNeueLTPro-UltLt;color:#FFFFFF;"
//                        "}"
//                        "QPushButton:hover{background-color:#E45E4C;}";
//    }
//    else if (WidgetStyle::themeColor == 1) {
//        for (int i = 0; i < 10; i++) {
//            btnNum[i]->setIcon(m_picturetowhite->drawSymbolicColoredPixmap(QPixmap(":/image/newIcon/standard/btnNum"+ QString::number(i) +".svg")));
//        }

//        // btnDZero->setIcon(QIcon(":/image/newIcon/standard/btnNum00.svg"));

//        btnClear->setIcon(m_picturetowhite->drawSymbolicColoredPixmap(QPixmap(":/image/newIcon/standard/btnClear.svg")));
//        btnDiv->setIcon(m_picturetowhite->drawSymbolicColoredPixmap(QPixmap(":/image/newIcon/standard/btnDiv.svg")));
//        btnMulti->setIcon(m_picturetowhite->drawSymbolicColoredPixmap(QPixmap(":/image/newIcon/standard/btnMulti.svg")));
//        btnDelete->setIcon(m_picturetowhite->drawSymbolicColoredPixmap(QPixmap(":/image/newIcon/standard/btnDelete.svg")));
//        btnSub->setIcon(m_picturetowhite->drawSymbolicColoredPixmap(QPixmap(":/image/newIcon/standard/btnSub.svg")));
//        btnAdd->setIcon(m_picturetowhite->drawSymbolicColoredPixmap(QPixmap(":/image/newIcon/standard/btnAdd.svg")));
//        btnEqual->setIcon(m_picturetowhite->drawSymbolicColoredPixmap(QPixmap(":/image/newIcon/standard/btnEqual.svg")));
//        btnPer->setIcon(m_picturetowhite->drawSymbolicColoredPixmap(QPixmap(":/image/newIcon/standard/btnPer.svg")));
//        btnPoint->setIcon(m_picturetowhite->drawSymbolicColoredPixmap(QPixmap(":/image/newIcon/standard/btnPoint.svg")));

//        btnStyle = "QPushButton{"
//                   "background-color:#27292C;border-radius:4px;"
//                   "font-size:42px;font-family:HelveticaNeueLTPro-UltLt;color:#FFFFFF;"
//                   "}"
//                   "QPushButton:hover{background-color:#474747;}";

//        btnOpStyle = "QPushButton{"
//                     "background-color:#222E36;border-radius:4px;"
//                     "font-size:35px;font-family:HelveticaNeueLTPro-UltLt;color:#FFFFFF;}"
//                     "QPushButton:hover{background-color:#474747;}";

//        btnEqualStyle = "QPushButton{"
//                        "background-color:#E64A19;border-radius:4px;"
//                        "font-size:48px;font-family:HelveticaNeueLTPro-UltLt;color:#FFFFFF;"
//                        "}"
//                        "QPushButton:hover{background-color:#E45E4C;}";
//    }
//    //布局按钮

//    for (int i = 0; i < 10; i++) {
//        btnNum[i]->setStyleSheet(btnStyle);
//    }

//    // btnDZero->setStyleSheet(btnStyle);

//    // 设置按钮样式standardModel
//    btnClear->setStyleSheet(btnStyle);
//    btnPer->setStyleSheet(btnStyle);
//    btnDelete->setStyleSheet(btnStyle);
//    btnPoint->setStyleSheet(btnStyle);

//    btnDiv->setStyleSheet(btnOpStyle);
//    btnMulti->setStyleSheet(btnOpStyle);
//    btnSub->setStyleSheet(btnOpStyle);
//    btnAdd->setStyleSheet(btnOpStyle);

//    btnEqual->setStyleSheet(btnEqualStyle);

//    this->setStyleSheet("border-radius:4px;");
}

/* 键盘响应事件 */
void ToolModelButton::keyPressEvent(QKeyEvent *event)
{
    switch (event->key())
    {
    case Qt::Key_Period:
        btnPoint->click();
        break;
    case Qt::Key_0:
        btnNum[0]->click();
        break;
    case Qt::Key_1:
        btnNum[1]->click();
        break;
    case Qt::Key_2:
        btnNum[2]->click();
        break;
    case Qt::Key_3:
        btnNum[3]->click();
        break;
    case Qt::Key_4:
        btnNum[4]->click();
        break;
    case Qt::Key_5:
        btnNum[5]->click();
        break;
    case Qt::Key_6:
        btnNum[6]->click();
        break;
    case Qt::Key_7:
        btnNum[7]->click();
        break;
    case Qt::Key_8:
        btnNum[8]->click();
        break;
    case Qt::Key_9:
        btnNum[9]->click();
        break;
    case Qt::Key_Plus:
        btnAdd->click();
        break;
    case Qt::Key_Minus:
        btnSub->click();
        break;
    case Qt::Key_Asterisk:
        btnMulti->click();
        break;
    case Qt::Key_Slash:
        btnDiv->click();
        break;
    case Qt::Key_Escape:
        btnClear->click();
        break;
    case Qt::Key_Percent:
        btnPer->click();
        break;
    case Qt::Key_Backspace:
        btnDelete->click();
        break;
    case Qt::Key_Enter:
        btnEqual->click();
        break;
    case 16777220:
        btnEqual->click();
        break;
    case 0x3d:
        btnEqual->click();
        break;
    default:
        return QWidget::keyPressEvent(event);
    }

    return;
}


// 换算器单位列表界面
UnitListWidget::UnitListWidget(QWidget *parent, QWidget *pWidget)
    : QWidget(parent)
{
    this->pWidget = pWidget;
    // 初始化组件
    this->setWidgetUi();

    // 设置组件样式
    this->setWidgetStyle();
}

// 初始化组件
void UnitListWidget::setWidgetUi()
{
    // 标题和取消
    unitTitle = new QLabel(this);
    btnCancel = new QPushButton(this);

    unitTitle->setText(tr("currency"));
//    unitTitle->setText("货币");
    unitTitle->setAlignment(Qt::AlignLeft);
    unitTitle->setFixedHeight(20);
    //unitTitle->resize(this->width(),20);

    btnCancel->setText(tr("cancel"));
//    btnCancel->setText("取消");

    btnCancel->setFixedSize(QSize(55, 20));
    //btnCancel->move(this->x() + 260 - btnCancel->width(), 15);

    // 搜索框
    //searchEdit = new QLineEdit(this);
    //searchEdit->setPlaceholderText(tr("search"));
//    searchEdit->setPlaceholderText("搜索");

    // 单位列表
    unitList = new QListWidget(this);
    unitList->setFrameShape(QListWidget::NoFrame);
    unitList->verticalScrollBar()->setProperty("drawScrollBarGroove", false);
    // unitList->setVerticalScrollMode(QListWidget::ScrollPerPixel);

    // 总体布局
    m_layout = new QHBoxLayout;
    m_layout->addSpacing(8);
    m_layout->addWidget(unitTitle);
    m_layout->addStretch();
    m_layout->addWidget(btnCancel);
    m_layout->addSpacing(8);
    m_layout->setMargin(0);
    m_layout->setSpacing(0);

    unitListLayout = new QVBoxLayout(this);
    unitListLayout->addLayout(m_layout);
    //unitListLayout->addWidget(searchEdit);
    unitListLayout->addWidget(unitList);

    unitWid = new QWidget(this);
    unitWid->setLayout(unitListLayout);
    unitWid->setObjectName("unitWid");

    QVBoxLayout *unitLayout = new QVBoxLayout(this);
    unitLayout->addWidget(unitWid);

    this->setLayout(unitLayout);
    this->setContentsMargins(0, 0, 0, 0);
    this->setFixedHeight(410);
    this->setFixedWidth(280);
    //this->resize(280,410);

    //btnCancel->raise();
    //searchEdit->hide();

    //绘制阴影
    QGraphicsDropShadowEffect *effect = new QGraphicsDropShadowEffect(this);
    static const QColor shadowColor = QColor(70,70,70); // 阴影颜色
    static const int shadowRedius = 10;                 // 阴影模糊度
    effect->setOffset(0, 0);                // (0,0)代表向四周发散
    effect->setColor(shadowColor);          // 设置阴影颜色
    effect->setBlurRadius(shadowRedius);    // 设定阴影的模糊半径，数值越大越模糊
    this->setGraphicsEffect(effect);
}

// 初始化组件样式
void UnitListWidget::setWidgetStyle()
{
    if (WidgetStyle::themeColor == 0) {
        unitTitle->setStyleSheet("width:30px;height:20px;font-size:14px;font-family:Source Han Sans CN;color:#000000;text-align:left;");
        btnCancel->setStyleSheet("width:30px;height:20px;border:0px;font-size:14px;"
                                 "font-family:Source Han Sans CN;color:#3D6BE5;");

        //searchEdit->setStyleSheet("width:246px;height:32px;background:rgba(231,231,231,0.5);border-radius:4px;");
        unitList->setStyleSheet("QListWidget{background-color:#FFFFFF;}"
                                "QListWidget::item{border-radius:4px;height:31px;}"
                                "QListWidget::item:selected{background-color:#3D6BE5;color:#FFFFFF;}"
                                "QListWidget::item:hover{background-color:#E9E9E9;}");

        // unitList->verticalScrollBar()->setStyleSheet("width:4px;background:#E9E9E9;border-radius:2px;border:0;");
        unitWid->setStyleSheet("#unitWid{background-color:#FFFFFF;border-radius:4px;}");

    } else if (WidgetStyle::themeColor == 1) {
        unitTitle->setStyleSheet("width:30px;height:20px;font-size:14px;font-family:Source Han Sans CN;color:#FFFFFF;text-align:left;");
        btnCancel->setStyleSheet("width:30px;height:20px;border:0px;font-size:14px;"
                                 "font-family:Source Han Sans CN;color:#EE8A18;");

        //searchEdit->setStyleSheet("width:246px;height:32px;background:rgba(30,30,32,0.5);border-radius:4px;");
        unitList->setStyleSheet("QListWidget{background-color:#121212;}"
                                "QListWidget::item{border-radius:4px;height:31px;}"
                                "QListWidget::item:selected{background-color:#FF800F;}"
                                "QListWidget::item:hover{background-color:#666666;}");

        // unitList->verticalScrollBar()->setStyleSheet("width:4px;background:#47474D;border-radius:2px;border:0;");
        unitWid->setStyleSheet("#unitWid{background-color:#121212;border-radius:4px;}");

    }
}

void UnitListWidget::setSelectItem(int row)
{
    m_item = unitList->item(row);
    for (int i = 0; i < unitList->count(); i++) {
        // 获取QWidget
        QWidget* itemWid = this->unitList->itemWidget(unitList->item(i));
        // 获取所有的QLabel
        QList<QLabel*> labelList = itemWid->findChildren<QLabel*>();
        if (i == row) {
            labelList[0]->setStyleSheet("color:#FFFFFF");
            labelList[1]->setStyleSheet("color:#FFFFFF");
        } else {
            if (WidgetStyle::themeColor == 0) {
                labelList[0]->setStyleSheet("color:#000000");
                labelList[1]->setStyleSheet("color:#000000");
            } else if (WidgetStyle::themeColor == 1) {
                labelList[0]->setStyleSheet("color:#FFFFFF");
                labelList[1]->setStyleSheet("color:#FFFFFF");
            }
        }

    }
}

// 失去焦点窗口隐藏
void UnitListWidget::focusOutEvent(QFocusEvent *e)
{
    if (this->btnCancel->hasFocus() || this->unitList->hasFocus() || this->pWidget->hasFocus()) {
        return ;
    }
    this->hide();
}

QListWidgetItem *UnitListWidget::getItem() const
{
    return m_item;
}

// 单位列表列表项
UnitListItem::UnitListItem(QWidget *parent)
    : QWidget(parent)
{
    // 设置列表项背景透明
    this->setAttribute(Qt::WA_TranslucentBackground, true);

    // 初始化列表项组件
    setWidgetUi();

    // 初始化列表项样式
    setWidgetStyle();
}

// 初始化列表项组件
void UnitListItem::setWidgetUi()
{
    // 初始化组件元素
    unitItemName = new QLabel(this);
    unitItemSymb = new QLabel(this);

    unitItemLayout = new QHBoxLayout(this);

    // 文字对齐方式和自适应放缩
    unitItemName->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    unitItemSymb->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);

//    unitItemName->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);

    // 将组件加入布局
    unitItemLayout->addSpacing(8);
    unitItemLayout->addWidget(unitItemName);
    unitItemLayout->addWidget(unitItemSymb);

    unitItemLayout->setSpacing(0);
    unitItemLayout->setMargin(0);

    this->setLayout(unitItemLayout);
}

// 初始化列表项样式
void UnitListItem::setWidgetStyle()
{
    // 单位名称组件样式
    QString itemNameStyle  = "font-family:SourceHanSansCN-ExtraLight;font-size:16px;background:transparent;";
    unitItemName->setStyleSheet(itemNameStyle);

    // 单位符号组件样式
    QString itemSymbStyle  = "font-family:SourceHanSansCN-ExtraLight;font-size:16px;width:30px;background:transparent;";
    unitItemSymb->setStyleSheet(itemSymbStyle);

    unitItemSymb->setFixedWidth(50);

    this->setFixedHeight(32);
}

void UpdateRateThread::run()
{
    // 访问汇算换算api
    QString strUrl = "https://api.exchangerate-api.com/v4/latest/CNY";
    QNetworkAccessManager manager;
    QNetworkRequest netRequest;
    QNetworkReply *netReply;
    QEventLoop loop;
    m_timer = new QTimer();
    connect(m_timer, SIGNAL(timeout()), &loop, SLOT(quit()));
    m_timer->start(10000);

    netRequest.setHeader(QNetworkRequest::ContentTypeHeader,"application/x-www-form-urlencoded");
    netRequest.setUrl(QUrl(strUrl));
    netReply = manager.get(netRequest);

    connect(netReply, SIGNAL(finished()), &loop, SLOT(quit()));
    loop.exec();

    m_timer->stop();

    if (netReply->error() != QNetworkReply::NoError) {
        return;
    }

    QString strRateAll = netReply->readAll();

    if (strRateAll == "") {
        return ;
    }

    QStringList strList;
    QByteArray rateJsonBuf = strRateAll.toUtf8();
    QJsonDocument rateJsonDoc = QJsonDocument::fromJson(rateJsonBuf);
    if(rateJsonDoc.isObject()){
        QJsonValue rateJsonValue = rateJsonDoc.object().value("rates");
        QJsonObject rateJsonObj = rateJsonValue.toObject();
        QStringList rateKeyList = rateJsonObj.keys();

        for (int i = 0; i < rateKeyList.size(); i++) {
            strList << rateKeyList.value(i);
            double rate = rateJsonObj.value(rateKeyList.value(i)).toDouble();
            strList << QString::number(rate);
        }
    }

    emit isDone(strList);

    manager.deleteLater();
    netReply->deleteLater();
    loop.deleteLater();
}
